;;;; -*- Mode: LISP; Syntax: ANSI-Common-Lisp; Base: 10 -*-

;;;; *************************************************************************
;;;; FILE IDENTIFICATION
;;;;
;;;; Name:          api.lisp
;;;; Purpose:       cl-digg Common Lisp API
;;;; Programmer:    Nicolas Lamirault <nicolas.lamirault@gmail.com>
;;;;
;;;; This file, part of cl-digg, is Copyright (c) 2007 by Nicolas Lamirault
;;;;
;;;; cl-digg are granted the rights to distribute and use this software
;;;; as governed by the terms of the Lisp Lesser GNU Public License
;;;; (http://opensource.franz.com/preamble.html), also known as the LLGPL.
;;;;
;;;; *************************************************************************


(in-package :cl-digg)


(defmacro with-digg-request ((request) choice &body body)
  "Macro with creates a HTTP request and executes BODY."
  `(let ((,request (concatenate 'string
                                *digg-server*
                                ,choice
                                "appkey="
                                *api-key*)))
     ,@body))


(defun digg-perform (url)
  "Make a query to the Digg web service.
If HTTP return code is 200, returns the body-stream of the HTTP request,
otherwise nil"
  (format t "Digg : ~A~%" url)
  (multiple-value-bind (body-stream status-code headers uri stream must-close)
      (drakma:http-request url)
    (declare (ignore headers uri stream must-close))
    (when (and status-code (= status-code 200))
      (format t "Digg HTTP Ok.")
      body-stream)))


(defun get-error-code (&optional code)
  "Get a list of all error codes and messages.
If CODE is not null, get the message for a specific error code."
(with-output-to-string (uri)
  (if code
      (format uri "~A~A?" *get-error-code* code)
      (format uri "~A?" *list-error-codes*))
  (with-digg-request (request) (get-output-stream-string uri)
    (digg-perform request))))


(defun get-users (&key user-name (sort "username-asc") (count 10) (offset 0))
  "Get all Digg users.
If USER-NAME is specified, get informations for a specific user.
SORT specify how to sort returned users. It could be username-asc: A-Z,
or username-desc: Z-A (the default).
COUNT is the number of users to retrieve (default 10).
OFFSET in the complete user list."
  (let (uri)
    (with-output-to-string (stream)
      (if user-name
          (format stream "~A~A" *get-user* user-name)
          (format stream "~A" *list-users*))
      (format stream "?sort=~A" sort)
      (format stream "&count=~A" count)
      (format stream "&offset=~A&" offset)
      (setf uri (get-output-stream-string stream)))
    (with-digg-request (request) uri
      (digg-perform request))))


(defun get-user-friends (user-name
                         &key friend-name (sort "username-asc") (count 10) (offset 0))
  "Get named user's friends or friend' informations if FRIEND-NAME is specified.
The first user name is the befriender, the second is the befriended.
SORT specify how to sort returned users. It could be username-asc: A-Z,
or username-desc: Z-A (the default).
COUNT is the number of users to retrieve (default 10).
OFFSET in the complete user list."
  (let (uri)
    (with-output-to-string (stream)
      (if friend-name
          (format stream "~A~A"
                  (format nil *get-user-friend* user-name)
                  friend-name)
          (format stream "~A" (format nil *list-user-friends* user-name)))
      (format stream "?sort=~A" sort)
      (format stream "&count=~A" count)
      (format stream "&offset=~A&" offset)
      (setf uri (get-output-stream-string stream)))
    (with-digg-request (request) uri
      (digg-perform request))))


(defun get-user-fans (user-name
                      &key fan-name (sort "username-asc") (count 10) (offset 0))
  "Get named user's fans or fan' informations if FAN-NAME is specified.
The first user name is the befriender, the second is the befriended.
SORT specify how to sort returned users. It could be username-asc: A-Z,
or username-desc: Z-A (the default).
COUNT is the number of users to retrieve (default 10).
OFFSET in the complete user list."
  (let (uri)
    (with-output-to-string (stream)
      (if fan-name
          (format stream "~A~A"
                  (format nil *get-user-fan* user-name)
                  fan-name)
          (format stream "~A" (format nil *list-user-fans* user-name)))
      (format stream "?sort=~A" sort)
      (format stream "&count=~A" count)
      (format stream "&offset=~A&" offset)
      (setf uri (get-output-stream-string stream)))
    (with-digg-request (request) uri
      (digg-perform request))))


(defun get-topics (&optional topic)
  "Get a list of all topics or the specified TOPIC."
  (let (uri)
    (with-output-to-string (stream)
      (if topic
          (format stream "~A~A?" *get-topic* topic)
          (format stream "~A?" *list-topics*))
      (setf uri (get-output-stream-string stream)))
    (with-digg-request (request) uri
      (digg-perform request))))



(defun list-stories (&key filter)
  "Get Digg stories. FILTER could be :upcoming or :popular."
  (let (uri)
    (with-output-to-string (stream)
      (cond ((equal filter :popular)
             (format stream "~A?" *get-popular-stories*))
            ((equal filter :upcoming)
             (format stream "~A?" *get-upcoming-stories*))
            ((null filter)
             (format stream "~A?" *get-stories*))
            (t (error "Unknown filter request ~A" filter)))
      (setf uri (get-output-stream-string stream)))
    (with-digg-request (request) uri
      (digg-perform request))))



(defun list-container-stories (container &key filter)
  "Get Digg stories for a given CONTAINER.
FILTER could be :upcoming or :popular."
  (let (uri)
    (with-output-to-string (stream)
      (cond ((equal filter :popular)
             (format stream "~A?"
                     (format nil *get-popular-container-stories* container)))
            ((equal filter :upcoming)
             (format stream "~A?"
                     (format nil *get-upcoming-container-stories* container)))
            ((null filter)
             (format stream "~A~A?" *get-container-stories* container))
            (t (error "Unknown filter request ~A" filter)))
      (setf uri (get-output-stream-string stream)))
    (with-digg-request (request) uri
      (digg-perform request))))


(defun list-topic-stories (topic &key filter)
  "Get Digg stories for a given TOPIC.
FILTER could be :upcoming or :popular."
  (let (uri)
    (with-output-to-string (stream)
      (cond ((equal filter :popular)
             (format stream "~A?"
                     (format nil *get-popular-topic-stories* topic)))
            ((equal filter :upcoming)
             (format stream "~A?"
                     (format nil *get-upcoming-topic-stories* topic)))
            ((null filter)
             (format stream "~A~A?" *get-topic-stories* topic))
            (t (error "Unknown filter request ~A" filter)))
      (setf uri (get-output-stream-string stream)))
    (with-digg-request (request) uri
      (digg-perform request))))


(defun list-user-stories (username)
  "Get Digg stories submitted by given user."
  (let (uri)
    (with-output-to-string (stream)
      (format stream "~A?" (format nil *get-user-stories* username))
      (setf uri (get-output-stream-string stream)))
    (with-digg-request (request) uri
      (digg-perform request))))


(defun get-story (&key ids title)
  "Get Digg stories. IDS is a list of Identifiant."
  (unless (or ids title)
    (error "Must defined IDS or TITLE."))
  (let (uri)
    (with-output-to-string (stream)
      (if ids
          (if (= 1 (length ids))
              (format stream "~A~A?" *get-story* (first ids))
              (format stream "~A/~{~A~^,~}?" *get-stories* ids))
          (format stream "~A~A?" *get-story* title))
      (setf uri (get-output-stream-string stream)))
    (with-digg-request (request) uri
      (digg-perform request))))

